#!/usr/bin/perl
use strict;
use warnings;
use YAML::XS qw/LoadFile/;

my $tmpl = <<'EOF';
# USE-style feature macros
#
# These macros define distribution-wide macros for conditional package
# build-time features.
#
# Usage:
# ./configure %{use_enable ipv6}
# ./configure %{use_enable X x11}
# ./configure %{use_enable
# ./configure %{use_with selinux}
# ./configure %{use_with pcre pcre2}
# ./configure %{use_with ncurses termcap tinfo}
# %if %{use nls}
#
# %use <feature>
# Indicates whether the <feature> is enabled, returns 0 or 1.
# Same as checking the %_with_<feature> macro directly but hides the
# implementation details and provides compatibility if the underscore macros
# ever get renamed.
#
# %use_enable <feature> [<name>] [<opt>]
# Returns --disable-<name> or --enable-<name>.
# <name> defaults to <feature> if not defined.  If <opt> is defined, it's
# passed as an argument to <name>.
#
# %use_with <feature> [<name>] [<opt>]
# Returns --without-<name> or --with-<name>.
# <name> defaults to <feature> if not defined.  If <opt> is defined, it's
# passed as an argument to <name>.
#
# Features are defined using %_use_<feature> macros.  The system-wide
# global settings can be found in this file.  Per-package local feature flags
# and overrides should be placed in macros.use.<package> and buildrequired by
# the package.  Machine-specific flags and overrides should go into ~/.rpmmacros
# or could be passed to rpmbuild with --define or --with/--without flags.

%use_generic() %{lua:
    local prefixon = rpm.expand('%1')
    local prefixoff = rpm.expand('%2')
    local feature = rpm.expand('%3')
    local name = rpm.expand('%{?4}') ~= '' and rpm.expand('%4') or feature
    local opt = rpm.expand('%{?5}')
    local rval
    if rpm.expand('%{_use_' .. feature .. '}') == '1' then
        rval = '--' .. prefixon .. '-' .. name
        if opt ~= '' then
            rval = rval .. '=' .. opt
        end
    else
        rval = '--' .. prefixoff .. '-' .. name
    end
    print(rval)
}
%use() %{expand:%%{?_use_%1}%%{!?_use_%1:0}}
%use_enable() %{use_generic enable disable %*}
%use_with() %{use_generic with without %*}
EOF

sub loadyaml {
    my ($file) = @_;
    my @data = sort {
        $a->{name} cmp $b->{name}
    } @{LoadFile($file)}
        or die("Cannot load ${file}!\n");
    return @data;
}

# <flag>:
#   enabled = 0|1
#   description = <string>
#   local:
#       <package>:
#           enabled = 0|1
#           description = <string>
my $definitions = '';
my %use = ();
for my $file ('global.yaml', <local.*.yaml>) {
    for my $def (loadyaml($file)) {
        $def->{name} =~ /^\w+$/
            or die("${file}: Flag " . ${def}->{name} . " is invalid!\n");
        if ($file eq 'global.yaml') {
            exists($use{$def->{name}})
                and die('Global flag ' . $def->{name} . " defined more than once!\n");
            $use{$def->{name}} = {
                enabled => $def->{enabled},
                description => $def->{description},
            };
        } else {
            my $pkg = ($file =~ s/^local\.(.+?)\.yaml$/$1/r);
            $use{$def->{name}} = {}
                unless exists($use{$def->{name}});
            $use{$def->{name}}->{local} = {}
                unless exists($use{$def->{name}}->{local});
            exists($use{$def->{name}}->{local}->{$pkg})
                and die('Local flag ' . $def->{name} . " for ${pkg} defined more than once!\n");
            $use{$def->{name}}->{local}->{$pkg}->{enabled} = $def->{enabled};
            $use{$def->{name}}->{local}->{$pkg}->{description} = $def->{description};
        }
    }
}
for my $flag (sort keys(%use)) {
    if ($use{$flag}->{description}) {
        $definitions .= "# ${flag}: $use{$flag}->{description}  "
            . ($use{$flag}->{enabled} ? 'Enabled' : 'Disabled')
            . ".\n";
    }
    if (exists($use{$flag}->{local})) {
        my (@localon, @localoff);
        for my $pkg (sort keys(%{$use{$flag}->{local}})) {
            $definitions .= "# ${flag}(${pkg}): $use{$flag}->{local}->{$pkg}->{description}  ";
            if ($use{$flag}->{local}->{$pkg}->{enabled}) {
                $definitions .= 'Enabled';
                push @localon, $pkg;
            } else {
                $definitions .= 'Disabled';
                push @localoff, $pkg;
            }
            $definitions .= ".\n";
        }
        $definitions .= "\%_use_${flag} \%{lua:\n";
        $definitions .= ' 'x4 . 'if ';
        if (@localon) {
            $definitions .= join("\n" . ' 'x8 . 'or ', map {
                    "rpm.expand(\"\%{name}\") == \"$_\""
                } @localon);
            $definitions .= " then \n"
                . ' 'x8
                . "print(rpm.expand(\"\%{bcond_without ${flag}}\%{with ${flag}}\"))\n";
        }
        $definitions .= ' 'x4 . 'elseif ' if (@localon && @localoff);
        if (@localoff) {
            $definitions .= join("\n" . ' 'x8 . 'or ', map {
                    "rpm.expand(\"\%{name}\") == \"$_\""
                } @localoff);
            $definitions .= " then \n"
                . ' 'x8
                . "print(rpm.expand(\"\%{bcond_with ${flag}}\%{with ${flag}}\"))\n";
        }
        $definitions .= ' 'x4 . "else\n"
            . ' 'x8
            . ($use{$flag}->{description}
                ? 'print(rpm.expand("%{bcond_with'
                . ($use{$flag}->{enabled} ? 'out' : '')
                . " ${flag}}\%{with ${flag}}\")"
                : 'print("0")')
            . "\n";
        $definitions .= ' 'x4 . "end\n}\n";
    } else {
        $definitions .= "\%_use_${flag} \%{bcond_with"
            . ($use{$flag}->{enabled} ? 'out' : '')
            . " ${flag}}\%{with ${flag}}\n";
    }
}
open(my $fh, '>', 'macros.use')
    or die("Cannot open macros.use for writing!\n");
print { $fh } "${tmpl}\n${definitions}";
close($fh)
    or die("Cannot close macros.use!\n");
